@@ -22,27 +22,27 @@
 #include <device.h>
 #include <fsp.h>
 
-/* The eeh event code will need updating if this is ever increased to
- * support more than 64 phbs */
-static struct phb *phbs[64];
+#define MAX_PHB_ID	256
+static struct phb *phbs[MAX_PHB_ID];
+int last_phb_id = 0;
 
 #define PCITRACE(_p, _bdfn, fmt, a...) \
-	prlog(PR_TRACE, "PHB%d:%02x:%02x.%x " fmt,	\
+	prlog(PR_TRACE, "PHB#%04x:%02x:%02x.%x " fmt,	\
 	      (_p)->opal_id,				\
 	      ((_bdfn) >> 8) & 0xff,			\
 	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
 #define PCIDBG(_p, _bdfn, fmt, a...) \
-	prlog(PR_DEBUG, "PHB%d:%02x:%02x.%x " fmt,	\
+	prlog(PR_DEBUG, "PHB#%04x:%02x:%02x.%x " fmt,	\
 	      (_p)->opal_id,				\
 	      ((_bdfn) >> 8) & 0xff,			\
 	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
 #define PCINOTICE(_p, _bdfn, fmt, a...) \
-	prlog(PR_NOTICE, "PHB%d:%02x:%02x.%x " fmt,	\
+	prlog(PR_NOTICE, "PHB#%04x:%02x:%02x.%x " fmt,	\
 	      (_p)->opal_id,				\
 	      ((_bdfn) >> 8) & 0xff,			\
 	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
 #define PCIERR(_p, _bdfn, fmt, a...) \
-	prlog(PR_ERR, "PHB%d:%02x:%02x.%x " fmt,	\
+	prlog(PR_ERR, "PHB#%04x:%02x:%02x.%x " fmt,	\
 	      (_p)->opal_id,				\
 	      ((_bdfn) >> 8) & 0xff,			\
 	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
@@ -162,7 +162,13 @@ static struct pci_device *pci_scan_one(struct phb *phb, struct pci_device *paren
 		goto fail;
 	}
 	pd->bdfn = bdfn;
+	pd->vdid = vdid;
+	pci_cfg_read32(phb, bdfn, PCI_CFG_SUBSYS_VENDOR_ID, &pd->sub_vdid);
+	pci_cfg_read32(phb, bdfn, PCI_CFG_REV_ID, &pd->class);
+	pd->class >>= 8;
+
 	pd->parent = parent;
+	list_head_init(&pd->pcrf);
 	list_head_init(&pd->children);
 	rc = pci_cfg_read8(phb, bdfn, PCI_CFG_HDR_TYPE, &htype);
 	if (rc) {
@@ -536,7 +542,7 @@ static uint8_t pci_scan(struct phb *phb, uint8_t bus, uint8_t max_bus,
 		 *    bridge (when we need to give aligned powers of two's
 		 *    on P7IOC). If is is set to false, we just adjust the
 		 *    subordinate bus number based on what we probed.
-		 *    
+		 *
 		 */
 		max_bus = save_max;
 		next_bus = phb->ops->choose_bus(phb, pd, next_bus,
@@ -787,28 +793,40 @@ static void pci_scan_phb(void *data)
 	pci_walk_dev(phb, pci_configure_mps, NULL);
 }
 
-int64_t pci_register_phb(struct phb *phb)
+int64_t pci_register_phb(struct phb *phb, int opal_id)
 {
-	int64_t rc = OPAL_SUCCESS;
-	unsigned int i;
-
-	/* This is called at init time in non-concurrent way, so no lock needed */
-	for (i = 0; i < ARRAY_SIZE(phbs); i++)
-		if (!phbs[i])
-			break;
-	if (i >= ARRAY_SIZE(phbs)) {
-		prerror("PHB: Failed to find a free ID slot\n");
-		rc = OPAL_RESOURCE;
+	/* The user didn't specify an opal_id, allocate one */
+	if (opal_id == OPAL_DYNAMIC_PHB_ID) {
+		/* This is called at init time in non-concurrent way, so no lock needed */
+		for (opal_id = 0; opal_id < ARRAY_SIZE(phbs); opal_id++)
+			if (!phbs[opal_id])
+				break;
+		if (opal_id >= ARRAY_SIZE(phbs)) {
+			prerror("PHB: Failed to find a free ID slot\n");
+			return OPAL_RESOURCE;
+		}
 	} else {
-		phbs[i] = phb;
-		phb->opal_id = i;
-		dt_add_property_cells(phb->dt_node, "ibm,opal-phbid",
-				      0, phb->opal_id);
-		PCIDBG(phb, 0, "PCI: Registered PHB\n");
+		if (opal_id >= ARRAY_SIZE(phbs)) {
+			prerror("PHB: ID %d out of range !\n", opal_id);
+			return OPAL_PARAMETER;
+		}
+		/* The user did specify an opal_id, check it's free */
+		if (phbs[opal_id]) {
+			prerror("PHB: Duplicate registration of ID %d\n", opal_id);
+			return OPAL_PARAMETER;
+		}
 	}
+
+	phbs[opal_id] = phb;
+	phb->opal_id = opal_id;
+	if (opal_id > last_phb_id)
+		last_phb_id = opal_id;
+	dt_add_property_cells(phb->dt_node, "ibm,opal-phbid", 0, phb->opal_id);
+	PCIDBG(phb, 0, "PCI: Registered PHB\n");
+
 	list_head_init(&phb->devices);
 
-	return rc;
+	return OPAL_SUCCESS;
 }
 
 int64_t pci_unregister_phb(struct phb *phb)
@@ -1141,6 +1159,8 @@ static void pci_add_slot_properties(struct phb *phb, struct pci_slot_info *info,
 	char loc_code[LOC_CODE_SIZE];
 	size_t base_loc_code_len = 0, slot_label_len = 0;
 
+	loc_code[0] = '\0';
+
 	if (phb->base_loc_code) {
 		base_loc_code_len = strlen(phb->base_loc_code);
 		strcpy(loc_code, phb->base_loc_code);
@@ -1188,10 +1208,15 @@ static void pci_add_loc_code(struct dt_node *np, struct pci_device *pd)
 	uint8_t class, sub;
 	uint8_t pos, len;
 
-	/* Look for a parent with a slot-location-code */
-	while (p && !blcode) {
-		blcode = dt_prop_get_def(p, "ibm,slot-location-code", NULL);
-		p = p->parent;
+	/* If there is a label assigned to the function, use it on openpower machines */
+	if (pd->slot_info && strlen(pd->slot_info->label) && !fsp_present()) {
+		blcode = pd->slot_info->label;
+	} else {
+		/* Look for a parent with a slot-location-code */
+		while (p && !blcode) {
+			blcode = dt_prop_get_def(p, "ibm,slot-location-code", NULL);
+			p = p->parent;
+		}
 	}
 	if (!blcode)
 		return;
@@ -1299,6 +1324,13 @@ static void pci_add_one_node(struct phb *phb, struct pci_device *pd,
 	uint32_t rev_class, vdid;
 	uint32_t reg[5];
 	uint8_t intpin;
+	const uint32_t ranges_direct[] = {
+				/* 64-bit direct mapping. We know the bridges
+				 * don't cover the entire address space so
+				 * use 0xf00... as a good compromise. */
+				0x02000000, 0x0, 0x0,
+				0x02000000, 0x0, 0x0,
+				0xf0000000, 0x0};
 
 	pci_cfg_read32(phb, pd->bdfn, 0, &vdid);
 	pci_cfg_read32(phb, pd->bdfn, PCI_CFG_REV_ID, &rev_class);
@@ -1320,7 +1352,7 @@ static void pci_add_one_node(struct phb *phb, struct pci_device *pd,
 	else
 		snprintf(name, MAX_NAME - 1, "%s@%x",
 			 cname, (pd->bdfn >> 3) & 0x1f);
-	np = dt_new(parent_node, name);
+	pd->dn = np = dt_new(parent_node, name);
 
 	/* XXX FIXME: make proper "compatible" properties */
 	if (pci_has_cap(pd, PCI_CFG_CAP_ID_EXP, false)) {
@@ -1349,8 +1381,8 @@ static void pci_add_one_node(struct phb *phb, struct pci_device *pd,
 	 *  - ...
 	 */
 
-	/* Add slot properties if needed */
-	if (pd->slot_info)
+	/* Add slot properties if needed and iff this is a bridge */
+	if (pd->slot_info && pd->is_bridge)
 		pci_add_slot_properties(phb, pd->slot_info, np);
 
 	/* Make up location code */
@@ -1366,7 +1398,6 @@ static void pci_add_one_node(struct phb *phb, struct pci_device *pd,
 
 	/* Print summary info about the device */
 	pci_print_summary_line(phb, pd, np, rev_class, cname);
-
 	if (!pd->is_bridge)
 		return;
 
@@ -1393,12 +1424,12 @@ static void pci_add_one_node(struct phb *phb, struct pci_device *pd,
 	 */
 	pci_std_swizzle_irq_map(np, pd, lstate, swizzle);
 
-	/* We do an empty ranges property for now, we haven't setup any
-	 * bridge windows, the kernel will deal with that
-	 *
-	 * XXX The kernel should probably fix that up
+	/* Parts of the OF address translation in the kernel will fail to
+	 * correctly translate a PCI address if translating a 1:1 mapping
+	 * (ie. an empty ranges property).
+	 * Instead add a ranges property that explicitly translates 1:1.
 	 */
-	dt_add_property(np, "ranges", NULL, 0);
+	dt_add_property(np, "ranges", ranges_direct, sizeof(ranges_direct));
 
 	list_for_each(&pd->children, child, link)
 		pci_add_one_node(phb, child, np, lstate, swizzle);
@@ -1418,6 +1449,17 @@ static void pci_add_nodes(struct phb *phb)
 		pci_add_one_node(phb, pd, phb->dt_node, lstate, 0);
 }
 
+static void pci_fixup_nodes(struct phb *phb)
+{
+	struct pci_device *pd;
+
+	if (!phb->ops->device_node_fixup)
+		return;
+
+	list_for_each(&phb->devices, pd, link)
+		phb->ops->device_node_fixup(phb, pd);
+}
+
 static void __pci_reset(struct list_head *list)
 {
 	struct pci_device *pd;
@@ -1448,9 +1490,11 @@ void pci_reset(void)
 
 static void pci_do_jobs(void (*fn)(void *))
 {
-	void *jobs[ARRAY_SIZE(phbs)];
+	struct cpu_job **jobs;
 	int i;
 
+	jobs = zalloc(sizeof(struct cpu_job *) * ARRAY_SIZE(phbs));
+	assert(jobs);
 	for (i = 0; i < ARRAY_SIZE(phbs); i++) {
 		if (!phbs[i]) {
 			jobs[i] = NULL;
@@ -1473,6 +1517,7 @@ static void pci_do_jobs(void (*fn)(void *))
 
 		cpu_wait_job(jobs[i], true);
 	}
+	free(jobs);
 }
 
 void pci_init_slots(void)
@@ -1495,6 +1540,14 @@ void pci_init_slots(void)
 			continue;
 		pci_add_nodes(phbs[i]);
 	}
+
+	/* Do device node fixups now that all the devices have been
+	 * added to the device tree. */
+	for (i = 0; i < ARRAY_SIZE(phbs); i++) {
+		if (!phbs[i])
+			continue;
+		pci_fixup_nodes(phbs[i]);
+	}
 }
 
 /*
@@ -1576,3 +1629,55 @@ void pci_restore_bridge_buses(struct phb *phb)
 {
 	pci_walk_dev(phb, __pci_restore_bridge_buses, NULL);
 }
+
+struct pci_cfg_reg_filter *pci_find_cfg_reg_filter(struct pci_device *pd,
+						   uint32_t start, uint32_t len)
+{
+	struct pci_cfg_reg_filter *pcrf;
+
+	/* Check on the cached range, which contains holes */
+	if ((start + len) <= pd->pcrf_start ||
+	    pd->pcrf_end <= start)
+		return NULL;
+
+	list_for_each(&pd->pcrf, pcrf, link) {
+		if (start >= pcrf->start &&
+		    (start + len) <= (pcrf->start + pcrf->len))
+			return pcrf;
+	}
+
+	return NULL;
+}
+
+struct pci_cfg_reg_filter *pci_add_cfg_reg_filter(struct pci_device *pd,
+						  uint32_t start, uint32_t len,
+						  uint32_t flags,
+						  pci_cfg_reg_func func)
+{
+	struct pci_cfg_reg_filter *pcrf;
+
+	pcrf = pci_find_cfg_reg_filter(pd, start, len);
+	if (pcrf)
+		return pcrf;
+
+	pcrf = zalloc(sizeof(*pcrf) + ((len + 0x4) & ~0x3));
+	if (!pcrf)
+		return NULL;
+
+	/* Don't validate the flags so that the private flags
+	 * can be supported for debugging purpose.
+	 */
+	pcrf->flags = flags;
+	pcrf->start = start;
+	pcrf->len = len;
+	pcrf->func = func;
+	pcrf->data = (uint8_t *)(pcrf + 1);
+
+	if (start < pd->pcrf_start)
+		pd->pcrf_start = start;
+	if (pd->pcrf_end < (start + len))
+		pd->pcrf_end = start + len;
+	list_add_tail(&pd->pcrf, &pcrf->link);
+
+	return pcrf;
+}
